==> tecfy_core_package-0.1.65.tar.gz: lib/services/sync.service.dart <==
part of tecfy_core_client_package;

class TecfySyncService {
  static bool isSyncing = false;
  static get isOfflineMode {
    return LocalConfigService.operationMode != OperationMode.online;
  }

  static List<String> _models;

  static Future<bool> reset() async {
    _models = null;
    return true;
  }

  static Future<int> getUnpostedCount() async {
    if (!isOfflineMode) return 0;
    // var unPosted = await DatabaseService.rawQuery(
    //     'select count(*) from Entity where isPosted = 0');
    // var deleted = await DatabaseService.rawQuery(
    //     'select count(*) from DeleteLog where isPosted = 0');
    // if (unPosted.length == 0 && deleted.length == 0) return -1;

    // return (unPosted.first.values.first + deleted.first.values.first);
    return (await getUnpostedRecords()).length;
  }

  static Future<List<dynamic>> getUnpostedRecords() async {
    var list = List.empty(growable: true);
    await _getModels();
    if (_models == null || _models.length == 0) {
      print('XXXXXXXXXXXXXXXX no Models');
    }
    var deleted = await DatabaseService.rawQuery(
        "select id,entity,timeStamp,postMessage,'Delete' as type from DeleteLog where isPosted = 0");
    list.addAll(deleted);

    for (var model in _models) {
      if (EntityService.getEntity(model).entityType == EntityTypes.SubEntity)
        continue;
      var result = await DatabaseService.rawQuery(
          "select id,'$model' as entity,timeStamp, postMessage,'Update' as type from $model where isPosted = 0");
      if (result != null && result.length > 0) {
        list.addAll(result);
      }
    }
    // var unPosted = await DatabaseService.rawQuery(
    //     "select id,entity,timeStamp,message,'Update' as type from Entity where isPosted = 0");
    // list.addAll(unPosted);
    list = jsonDecode(jsonEncode(list));
    for (var rec in list) {
      if (rec['postMessage'] != null && rec['postMessage'].length > 10) {
        var err = rec['postMessage'].toString().split('InnerError:');
        rec['postMessage'] = err[0];
        if (err.length > 1) rec['innerError'] = err[1];
      }
    }
    return list;
  }

  static Future<List<dynamic>> getLocalRecords() async {
    await _getModels();
    if (_models == null || _models.length == 0) {
      print('XXXXXXXXXXXXXXXX no Models');
    }
    List<dynamic> localEntities = [];
    for (var model in _models) {
      var result = await DatabaseService.rawQuery(
          "select count(*) as count from $model");
      if (result != null && result.length > 0) {
        localEntities.add({
          'entity': model,
          'entityName': EntityService.getEntity(model).name,
          'count': result.first.values.first
        });
      } else {
        localEntities.add({
          'entity': model,
          'entityName': EntityService.getEntity(model).name,
          'count': -1
        });
      }
    }
    localEntities.sort((a, b) => a['entityName'].compareTo(b['entityName']));
    return localEntities;
  }

  static Future<bool> undoDelete(String id) async {
    var result = await DatabaseService.delete('DeleteLog',
        where: 'id = ?', whereArgs: [id]);
    return result > 0;
  }

  static Future<bool> pushNow() async {
    try {
      var isLoggin = await AccountService.isLogin(null);
      if (!ConnectionService.isOfflineMode || isLoggin.data != true) {
        return true;
      }
      print('++++++++++++++++++++++++++++++++ Start Sync');

      //====================================================
      //======  delete local
      //====================================================
      print('============= delete local');
      var lastSyncDeletedInt = ConfigService.getValueInt('lastSyncDeleted');
      DateTime lastSyncDeleted = DateTime.now().toUtc();
      if (lastSyncDeletedInt != null && lastSyncDeletedInt > 0)
        lastSyncDeleted =
            DateTime.fromMillisecondsSinceEpoch(lastSyncDeletedInt);
      lastSyncDeleted = lastSyncDeleted.add(Duration(hours: -5));
      var serverDeleted = await TecfyBasicApp.apiService.httpGetDynamic(
          'sync/getDeleted/${intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(lastSyncDeleted)}');
      if (serverDeleted.data != null && serverDeleted.data.length > 0) {
        var idsToDelete =
            serverDeleted.data.map<String>((e) => e['id'].toString()).toList();
        print(idsToDelete);
        await DatabaseService.deleteEntityBulk(idsToDelete);

        (serverDeleted.data as List<dynamic>)
            .sort((a, b) => b['timeStamp'].compareTo(a['timeStamp']));

        ConfigService.setValueInt(
            'lastSyncDeleted',
            BaseModel.parseUTC(serverDeleted.data.first['timeStamp'])
                .millisecondsSinceEpoch);
      }
      //====================================================
      //======  delete from server
      //====================================================
      print('============= delete from server');
      var localDeleted =
          (await DatabaseService.query('DeleteLog', where: 'isPosted = 0'))
              .toList();

      if (localDeleted.length > 0) {
        var sendServerDeleteResult = await TecfyBasicApp.apiService.httpPost(
            'sync/bulkDelete',
            localDeleted.map((e) {
              return {'id': e['id'], 'entity': e['entity']};
            }).toList(),
            true);

        print('+++++++++++++++++++++++++++++++++++ sendServerDeleteResult');
        print(sendServerDeleteResult.data);
        if (sendServerDeleteResult.success &&
            sendServerDeleteResult.data != null &&
            sendServerDeleteResult.data.length > 0) {
          print(sendServerDeleteResult.data);
          var deleteResult = sendServerDeleteResult.data as List<dynamic>;
          await DatabaseService.exeBatch((b) async {
            for (var i in deleteResult) {
              if (i['success']) {
                b.delete('DeleteLog', where: 'id = ?', whereArgs: [i['id']]);
              } else {
                b.update(
                    'DeleteLog', {'postMessage': 'Item related to other items'},
                    where: 'id = ?', whereArgs: [i['id']]);
              }
            }
            b.commit();
            return true;
          });
        }
      }
      //====================================================
      //======  Post to server
      //====================================================
      // var unPosted =
      //     (await DatabaseService.query('Entity', where: 'isPosted = 0'))
      //         .toList();
      // print('SSSSSSSSSSSSS-------------- unPosted to sync ${unPosted.length}');
      // if (unPosted.length == 0) return true;

      //===========
      //== Post un-posted items
      //===========
      await _getModels();

      for (var model in _models) {
        model = EntityService.lowerFirst(model);
        var entity = EntityService.getEntity(model);
        if (entity.entityType == EntityTypes.SubEntity) continue;
        var unPosted =
            (await DatabaseService.query(model, where: 'isPosted = 0'))
                .toList();
        if (unPosted.length > 0) {
          print('Found ${unPosted.length} items');
        }
        var isHasSubEntity = entity.properties
            .any((p) => p.propertyType == PropertyTypes.OtherEntityMany);

        for (var item in unPosted) {
          Map<String, dynamic> finalItem;
          if (isHasSubEntity) {
            finalItem = await EntityService.getOne(model, item['id'],
                withRelatedData: true);
          } else
            finalItem = DatabaseService.fromDatabaseEntity(entity, item);
          //print(finalItem);
          var result = await TecfyBasicApp.apiService.httpPostEx(
              'entity/addupdate/$model',
              jsonEncode(finalItem, toEncodable: EntityService.customEncode)
                  .replaceAll('\"', '\''));
          if (result.success) {
            await DatabaseService.update(
                model, {'isPosted': 1, 'postMessage': ''},
                where: 'id = ?', whereArgs: [item['id']], skipDataUpdate: true);
          } else {
            await DatabaseService.update(
                model,
                {
                  'postMessage':
                      '${result.message} InnerError:${result.innerError}'
                },
                where: 'id = ?',
                whereArgs: [item['id']],
                skipDataUpdate: true);
            print(result.toString());
          }
        }
      }
      return true;
    } catch (err, stack) {
      print('XXXXXXXXXXXXXXXXXXXXXXXXXXXX Error push nowXXXXXXX');
      print(err);
      print(stack);
      return false;
    }
  }

  static _getModels() async {
    if (_models != null && _models.length > 0) return;
    var modelsResult =
        await TecfyBasicApp.apiService.httpGetDynamic('Sync/getModels');
    if (!modelsResult.success) {
      var val = ConfigService.getValueString('syncModels');
      if (val != null) {
        modelsResult.data = jsonDecode(val);
      }
    } else {
      ConfigService.setValueString('syncModels', jsonEncode(modelsResult.data));
    }
    if (modelsResult.data == null) return _models = [];
    _models = modelsResult.data
        .map<String>((e) => EntityService.lowerFirst(e.toString()))
        .where((d) =>
            d != 'attachment' &&
            d != 'branchUser' &&
            d != 'tecfyScreenPermission' &&
            d != 'tecfySecurityGroup' &&
            d != 'tecfyUser')
        .toList();
  }

  static Future<bool> syncNow() async {
    if (ConnectionService.isOnlineMode) return true;
    bool isAllSuccess = true;
    isAllSuccess = await pushNow();

    await _getModels();

    // var lastSyncEntityInt = ConfigService.getValueInt('lastSyncEntity');

    DateTime lastSyncEntity = DateTime.now().toUtc();
    switch (LocalConfigService.operationMode) {
      case OperationMode.offline_cache_3months:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 3));
        break;
      case OperationMode.offline_cache_6months:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 6));
        break;
      case OperationMode.offline_cache_12months:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 12));
        break;
      case OperationMode.offline_cache_all:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 60));
        break;
      default:
    }
    lastSyncEntity = lastSyncEntity.add(Duration(days: -1));
    // if (lastSyncEntityInt != null && lastSyncEntityInt > 0)
    //   lastSyncEntity = DateTime.fromMillisecondsSinceEpoch(lastSyncEntityInt);
    // var lastTime = DateTime.now().toUtc();
    List<dynamic> syncRequest = [];
    int index = 1;
    for (var model in _models) {
      if (ConnectionService.isOnlineMode) return true;
      var entity = EntityService.getEntity(model);
      var lastLocal = await DatabaseService.query(model,
          where: 'isPosted= 1',
          orderBy: 'timeStamp desc',
          limit: 1,
          columns: ['timeStamp']);
      var timeAfter = lastSyncEntity;
      if (lastLocal != null &&
          lastLocal.length > 0 &&
          lastLocal?.first?.values?.first != null) {
        timeAfter =
            DateTime.fromMillisecondsSinceEpoch(lastLocal?.first?.values?.first)
                .add(Duration(hours: -10));
      } else if (entity.entityType == EntityTypes.Lookup) {
        timeAfter = lastSyncEntity.add(Duration(days: -360 * 50));
      }
      syncRequest.add({
        'entityName': model,
        'lastEntity': intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(timeAfter)
      });
    }

    var syncResponse = await TecfyBasicApp.apiService
        .httpPost('sync/syncRequest', syncRequest);
    index = 0;
    print('----- Got ${syncResponse.data.length} new records');

    for (var res in syncResponse.data) {
      index++;
      if (res['data'] == null || res['data'].length == 0) continue;
      var req = syncRequest.firstWhere(
          (e) => e['entityName'] == res['entityName'],
          orElse: () => null);
      if (req == null) continue;
      //   await DatabaseService.delete(res['entityName'],
      //       where: 'isPosted= 1 and timeStamp > ?',
      //       whereArgs: [
      //         BaseModel.parseUTC(req['lastEntity']).millisecondsSinceEpoch
      //       ]);
      var oldData = await DatabaseService.query(res['entityName'],
          where: 'isPosted= 0', columns: ['id']);
      var oldIds = oldData.map((d) => d['id']).toList();
      var result = await DatabaseService.addEntityAll(res['entityName'],
          res['data'].where((r) => !oldIds.contains(r['id'])).toList(),
          isPosted: true);
      if (!result) isAllSuccess = false;
      print(
          '$index/${syncResponse.data.length} --- old:${oldData.length}  new:${res['data'].length} --- $result');
    }
    // ConfigService.setValueInt(
    //     'lastSyncEntity', lastTime.millisecondsSinceEpoch);
    //===========
    //== Load Item category
    //===========
    // var categoryResult =
    //     await TecfyBasicApp.apiService.httpGetDynamic('pos/GetItemCategorySecondry');

    // if (categoryResult.success) {
    //   var items = (categoryResult.data as List<dynamic>)
    //       .map<ItemCategory>((e) => ItemCategory.fromJson(e, null))
    //       .toList();

    //   await DatabaseService.delete('ItemCategory');

    //   await DatabaseService.exeBatch((batch) async {
    //     for (var item in items) {
    //       batch.insert('ItemCategory', item.toJson());
    //     }
    //     return true;
    //   });
    //   print(
    //       '+++++++++++++++++++ ItemCategory count ${(await DatabaseService.rawQuery('select count(*) from ItemCategory'))}');
    // } else {
    //   print('XXXX Error ItemCategory ${categoryResult.message} ');
    // }

    return isAllSuccess;
  }
}

==> tecfy_core_package-0.1.81.tar.gz: lib/services/sync.service.dart <==
part of tecfy_core_client_package;

class TecfySyncService {
  static bool isSyncing = false;
  static get isOfflineMode {
    return LocalConfigService.operationMode != OperationMode.online;
  }

  static List<String> _models;

  static Future<bool> reset() async {
    _models = null;
    return true;
  }

  static Future<int> getUnpostedCount() async {
    if (!isOfflineMode) return 0;
    // var unPosted = await DatabaseService.rawQuery(
    //     'select count(*) from Entity where isPosted = 0');
    // var deleted = await DatabaseService.rawQuery(
    //     'select count(*) from DeleteLog where isPosted = 0');
    // if (unPosted.length == 0 && deleted.length == 0) return -1;

    // return (unPosted.first.values.first + deleted.first.values.first);
    return (await getUnpostedRecords()).length;
  }

  static Future<List<dynamic>> getUnpostedRecords() async {
    var list = List.empty(growable: true);
    await _getModels();
    if (_models == null || _models.length == 0) {
      print('XXXXXXXXXXXXXXXX no Models');
    }
    var deleted = await DatabaseService.rawQuery(
        "select id,entity,timeStamp,postMessage,'Delete' as type from DeleteLog where isPosted = 0");
    list.addAll(deleted);

    for (var model in _models) {
      if (EntityService.getEntity(model).entityType == EntityTypes.SubEntity)
        continue;
      var result = await DatabaseService.rawQuery(
          "select id,'$model' as entity,timeStamp, postMessage,'Update' as type from $model where isPosted = 0");
      if (result != null && result.length > 0) {
        list.addAll(result);
      }
    }
    // var unPosted = await DatabaseService.rawQuery(
    //     "select id,entity,timeStamp,message,'Update' as type from Entity where isPosted = 0");
    // list.addAll(unPosted);
    list = jsonDecode(jsonEncode(list));
    for (var rec in list) {
      if (rec['postMessage'] != null && rec['postMessage'].length > 10) {
        var err = rec['postMessage'].toString().split('InnerError:');
        rec['postMessage'] = err[0];
        if (err.length > 1) rec['innerError'] = err[1];
      }
    }
    return list;
  }

  static Future<List<dynamic>> getLocalRecords() async {
    await _getModels();
    if (_models == null || _models.length == 0) {
      print('XXXXXXXXXXXXXXXX no Models');
    }
    List<dynamic> localEntities = [];
    for (var model in _models) {
      var result = await DatabaseService.rawQuery(
          "select count(*) as count from $model");
      if (result != null && result.length > 0) {
        localEntities.add({
          'entity': model,
          'entityName': EntityService.getEntity(model).name,
          'count': result.first.values.first
        });
      } else {
        localEntities.add({
          'entity': model,
          'entityName': EntityService.getEntity(model).name,
          'count': -1
        });
      }
    }
    localEntities.sort((a, b) => a['entityName'].compareTo(b['entityName']));
    return localEntities;
  }

  static Future<bool> undoDelete(String id) async {
    var result = await DatabaseService.delete('DeleteLog',
        where: 'id = ?', whereArgs: [id]);
    return result > 0;
  }

  static Future<bool> pushNow() async {
    try {
      var isLoggin = await AccountService.isLogin(null);
      if (!ConnectionService.isOfflineMode || isLoggin.data != true) {
        return true;
      }
      print('++++++++++++++++++++++++++++++++ Start Sync');

      //====================================================
      //======  delete local
      //====================================================
      print('============= delete local');
      var lastSyncDeletedInt = ConfigService.getValueInt('lastSyncDeleted');
      DateTime lastSyncDeleted = DateTime.now().toUtc();
      if (lastSyncDeletedInt != null && lastSyncDeletedInt > 0)
        lastSyncDeleted =
            DateTime.fromMillisecondsSinceEpoch(lastSyncDeletedInt);
      lastSyncDeleted = lastSyncDeleted.add(Duration(hours: -5));
      var serverDeleted = await TecfyBasicApp.apiService.httpGetDynamic(
          'sync/getDeleted/${intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(lastSyncDeleted)}');
      if (serverDeleted.data != null && serverDeleted.data.length > 0) {
        var idsToDelete =
            serverDeleted.data.map<String>((e) => e['id'].toString()).toList();
        print(idsToDelete);
        await DatabaseService.deleteEntityBulk(idsToDelete);

        (serverDeleted.data as List<dynamic>)
            .sort((a, b) => b['timeStamp'].compareTo(a['timeStamp']));

        ConfigService.setValueInt(
            'lastSyncDeleted',
            BaseModel.parseUTC(serverDeleted.data.first['timeStamp'])
                .millisecondsSinceEpoch);
      }
      //====================================================
      //======  delete from server
      //====================================================
      print('============= delete from server');
      var localDeleted =
          (await DatabaseService.query('DeleteLog', where: 'isPosted = 0'))
              .toList();

      if (localDeleted.length > 0) {
        var sendServerDeleteResult = await TecfyBasicApp.apiService.httpPost(
            'sync/bulkDelete',
            localDeleted.map((e) {
              return {'id': e['id'], 'entity': e['entity']};
            }).toList(),
            true);

        print('+++++++++++++++++++++++++++++++++++ sendServerDeleteResult');
        print(sendServerDeleteResult.data);
        if (sendServerDeleteResult.success &&
            sendServerDeleteResult.data != null &&
            sendServerDeleteResult.data.length > 0) {
          print(sendServerDeleteResult.data);
          var deleteResult = sendServerDeleteResult.data as List<dynamic>;
          await DatabaseService.exeBatch((b) async {
            for (var i in deleteResult) {
              if (i['success']) {
                b.delete('DeleteLog', where: 'id = ?', whereArgs: [i['id']]);
              } else {
                b.update(
                    'DeleteLog', {'postMessage': 'Item related to other items'},
                    where: 'id = ?', whereArgs: [i['id']]);
              }
            }
            b.commit();
            return true;
          });
        }
      }
      //====================================================
      //======  Post to server
      //====================================================
      // var unPosted =
      //     (await DatabaseService.query('Entity', where: 'isPosted = 0'))
      //         .toList();
      // print('SSSSSSSSSSSSS-------------- unPosted to sync ${unPosted.length}');
      // if (unPosted.length == 0) return true;

      //===========
      //== Post un-posted items
      //===========
      await _getModels();

      for (var model in _models) {
        model = EntityService.lowerFirst(model);
        var entity = EntityService.getEntity(model);
        if (entity.entityType == EntityTypes.SubEntity) continue;
        var unPosted =
            (await DatabaseService.query(model, where: 'isPosted = 0'))
                .toList();
        if (unPosted.length > 0) {
          print('Found ${unPosted.length} items');
        }
        var isHasSubEntity = entity.properties
            .any((p) => p.propertyType == PropertyTypes.OtherEntityMany);

        for (var item in unPosted) {
          Map<String, dynamic> finalItem;
          if (isHasSubEntity) {
            finalItem = await EntityService.getOne(model, item['id'],
                withRelatedData: true);
          } else
            finalItem = DatabaseService.fromDatabaseEntity(entity, item);
          //print(finalItem);
          var result = await TecfyBasicApp.apiService.httpPostEx(
              'entity/addupdate/$model',
              jsonEncode(finalItem, toEncodable: EntityService.customEncode)
                  .replaceAll('\"', '\''));
          if (result.success) {
            await DatabaseService.update(
                model, {'isPosted': 1, 'postMessage': ''},
                where: 'id = ?', whereArgs: [item['id']], skipDataUpdate: true);
          } else {
            await DatabaseService.update(
                model,
                {
                  'postMessage':
                      '${result.message} InnerError:${result.innerError}'
                },
                where: 'id = ?',
                whereArgs: [item['id']],
                skipDataUpdate: true);
            print(result.toString());
          }
        }
      }
      return true;
    } catch (err, stack) {
      print('XXXXXXXXXXXXXXXXXXXXXXXXXXXX Error push nowXXXXXXX');
      print(err);
      print(stack);
      return false;
    }
  }

  static _getModels() async {
    if (_models != null && _models.length > 0) return;
    var modelsResult =
        await TecfyBasicApp.apiService.httpGetDynamic('Sync/getModels');
    if (!modelsResult.success) {
      var val = ConfigService.getValueString('syncModels');
      if (val != null) {
        modelsResult.data = jsonDecode(val);
      }
    } else {
      ConfigService.setValueString('syncModels', jsonEncode(modelsResult.data));
    }
    if (modelsResult.data == null) return _models = [];
    _models = modelsResult.data
        .map<String>((e) => EntityService.lowerFirst(e.toString()))
        .where((d) =>
            d != 'attachment' &&
            d != 'branchUser' &&
            d != 'tecfyScreenPermission' &&
            d != 'tecfySecurityGroup' &&
            d != 'tecfyUser')
        .toList();
  }

  static Future<bool> syncNow() async {
    if (ConnectionService.isOnlineMode) return true;
    bool isAllSuccess = true;
    isAllSuccess = await pushNow();

    await _getModels();

    // var lastSyncEntityInt = ConfigService.getValueInt('lastSyncEntity');

    DateTime lastSyncEntity = DateTime.now().toUtc();
    switch (LocalConfigService.operationMode) {
      case OperationMode.offline_cache_3months:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 3));
        break;
      case OperationMode.offline_cache_6months:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 6));
        break;
      case OperationMode.offline_cache_12months:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 12));
        break;
      case OperationMode.offline_cache_all:
        lastSyncEntity = lastSyncEntity.add(Duration(days: -30 * 60));
        break;
      default:
    }
    lastSyncEntity = lastSyncEntity.add(Duration(days: -1));
    // if (lastSyncEntityInt != null && lastSyncEntityInt > 0)
    //   lastSyncEntity = DateTime.fromMillisecondsSinceEpoch(lastSyncEntityInt);
    // var lastTime = DateTime.now().toUtc();
    int recordReceived = 0;
    do {
      List<dynamic> syncRequest = [];
      int index = 1;
      for (var model in _models) {
        if (ConnectionService.isOnlineMode) return true;
        var entity = EntityService.getEntity(model);
        var lastLocal = await DatabaseService.query(model,
            where: 'isPosted= 1',
            orderBy: 'timeStamp desc',
            limit: 1,
            columns: ['timeStamp', 'id']);
        var timeAfter = lastSyncEntity;
        String lastItemId = null;
        int lastTimeMillisecond = 0;
        if (lastLocal != null &&
            lastLocal.length > 0 &&
            lastLocal?.first?.values?.first != null) {
          timeAfter = DateTime.fromMillisecondsSinceEpoch(
              lastLocal?.first?.values?.first);
          lastItemId = lastLocal?.first?.values?.last;
          lastTimeMillisecond = lastLocal?.first?.values?.first;
          //.add(Duration(hours: -10))
        } else if (entity.entityType == EntityTypes.Lookup) {
          timeAfter = lastSyncEntity.add(Duration(days: -360 * 50));
        }
        syncRequest.add({
          'entityName': model,
          'lastItemId': lastItemId,
          'lastEntityM': lastTimeMillisecond,
          'lastEntity':
              intl.DateFormat("yyyy-MM-ddTHH:mm:ss.S").format(timeAfter)
        });
        print(
            '------ $model ---- $lastItemId -----  ${intl.DateFormat("yyyy-MM-ddTHH:mm:ss.S").format(timeAfter)}');
      }

      var syncResponse = await TecfyBasicApp.apiService
          .httpPost('sync/syncRequest?limit=5000', syncRequest);
      index = 0;
      if (syncResponse.data == null) {
        return false;
      }
      recordReceived = 0;
      for (var e in syncResponse.data) recordReceived += e['data'].length;
      print('----- Got ${recordReceived} new records');

      for (var res in syncResponse.data) {
        index++;
        if (res['data'] == null || res['data'].length == 0) continue;
        var req = syncRequest.firstWhere(
            (e) => e['entityName'] == res['entityName'],
            orElse: () => null);
        if (req == null) continue;
        var oldData = await DatabaseService.query(res['entityName'],
            where: 'isPosted= 0', columns: ['id']);
        var oldIds = oldData.map((d) => d['id']).toList();
        var result = await DatabaseService.addEntityAll(res['entityName'],
            res['data'].where((r) => !oldIds.contains(r['id'])).toList(),
            isPosted: true);
        if (!result) isAllSuccess = false;
        print(
            '$index/${syncResponse.data.length} --- old:${oldData.length}  new:${res['data'].length} --- $result');
      }
    } while (recordReceived == 5000);
    return isAllSuccess;
  }
}
